\\ Copyright 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("dragon-defines.gp");

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}



\\-----------------------------------------------------------------------------

\\ Michel Rigo, "Generalization of Automatic Sequences For Numeration
\\ Systems On a Regular Language", Theoretical Computer Science, volume 244,
\\ 2000, pages 271-281.  https://doi.org/10.1016/S0304-3975(00)00163-8

\\ cf section 5 example 1 another
\\ not in OEIS: 0,1,2,4,5,6,7,9,10,11,12,14,15,16,17,21,22,23,25,27,28


\\-----------------------------------------------------------------------------
\\ A317948   automaton  a*b* fed into ...

\\ by ab strings and state machine
A317948_vector(len) =
{
  my(a=[1,2,3,0], b=[0,3,1,2]);
  my(v=[0], ret=v);
  while(#v<len,
    v = concat(a[v[1]+1],            \\ all a's
               apply(x->b[x+1],v));  \\ additional b
    ret=concat(ret,v));
  Vec(ret,len);
}
{
  check_equal(A317948_vector(#A317948_samples),
              A317948_samples,
              "A317948_vector() vs A317948_samples");
}

check_equal(Vecsmall([1,4,2,3])^-1, Vecsmall([1,3,4,2]))
\\                                            0 2 3 1
\\ 0,
\\ 1, 0,
\\ 2, 3, 0,
\\ 3, 1, 2, 0,
\\ 0, 2, 3, 1, 0,
\\ 1, 0, 1, 2, 3, 0
\\ my(k=30,v=A317948_vector(k*(k+1)/2),w=1,upto=w); \
\\ for(i=1,#v, print1(v[i],","); upto--; if(upto==0,upto+=w++;print()));

p(n) = [0,2,3,1, 0,1,2,3, 0,3,1,2][n+1];
\\ not in OEIS: 0,2,3,1, 0,1,2,3, 0,3,1,2

T(d,k) =
{
  d>=0 || error();
  k>=0 || error();
  p((3*d+k)%12);
}
A317948_vector_by_T(len) =
{
  my(ret=vector(len), upto=0);
  for(d=0,len,
     for(k=0,d,
        if(upto>=len,break(2));
        ret[upto++] = T(d,k)));
  ret;
}
{
  check_equal(A317948_vector_by_T(500),
              A317948_vector     (500),
              "T vs A317948_vector");

  check_equal(matrix(100,100,d,k,d--;k--; if(k<=d-12, T(d,k))),
              matrix(100,100,d,k,d--;k--; if(k<=d-12, T(d,k+12))),
              "T rows 12-periodic");
}

S(m,k) =
{
  m>=0 || error();
  k>=0 || error();
  if(m%4==0, 0, (((m % 4) - k - 1) % 3) + 1);
}
{
  check_equal(matrix(100,100,m,k,m--;k--; if(m%=4, (m-k-1)%3+1, 0) ),
              matrix(100,100,m,k,m--;k--; S(m,k)),
              "S compact form");

  check_equal(matrix(100,100,m,k,m--;k--; S(m,k)),
              matrix(100,100,m,k,m--;k--; S(m,k+3)),
              "S rows 3-periodic");
  check_equal(matrix(100,100,m,k,m--;k--; S(m,k)),
              matrix(100,100,m,k,m--;k--; S(m+4,k)),
              "S columns 4-periodic");

  check_equal(matrix(20,20,d,k,d--;k--; if(k<=d, S(d-k,k))),
              matrix(20,20,d,k,d--;k--; if(k<=d, T(d,k))),
              "S vs T");

  check_equal(antidiagonals_of_func(500,S,0,0),
              A317948_vector(500),
              "S antidiagonals_of_func");
}


{
  for(d=0,7, for(k=0,d, print1(T(d,k),", ")); print());
  print();
  my(d=7); for(m=0,d, for(k=0,d-m, print1(S(m,k),", ")); print());
}
\\ {
\\   my(d=8); for(m=0,d, for(k=0,d-m,
\\                          my(n=mk_to_n(m,k), r,s=sqrtint(8*n+1,&r));
\\                          print1([s,r],", ")); print());
\\ }
\\ antidiagonals_of_func(15, mk_to_n, 0, 0)
\\ antidiagonals_of_func(15, (x,y)->my(n=mk_to_n(x,y),r,s=sqrtint(8*n+1,&r)); [s,r], 0, 0)

mk_to_n(x,y) = ((x+y)^2 + 3*y + x)/2 + 1;
check_equal(mk_to_n(0,0), 1, "mk_to_n(0,0)");
check_equal(mk_to_n(1,0), 2);
check_equal(mk_to_n(0,1), 3);
check_equal(mk_to_n(2,0), 4);

n_to_d(n) = n--; floor( (sqrtint(8*n+1) - 1)/2 );
n_to_k(n) = n--; my(d = floor( (sqrtint(8*n+1) - 1)/2 )); n - d*(d+1)/2;
n_to_m(n) = n_to_d(n) - n_to_k(n);
{
  check_equal(vector(100,n, mk_to_n(n_to_m(n), n_to_k(n))),
              vector(100,n, n),
              "mk_to_n() round trip");
}

a(n) = T(n_to_d(n), n_to_k(n));
{
  check_equal(vector(#A317948_samples,n,a(n)),
              A317948_samples,
              "a() vs A317948_samples");

  check_equal(vector(500,n,a(n)),
              A317948_vector(500),
              "a() vs A317948_vector()");
}

\\-----------------------------------------------------------------------------
\\ where 0,0

Triangular(n) = n*(n+1)/2;
A033585(n) = Triangular(4*n);

{
  check_equal(vector(#A033585_samples,n,n--; Triangular(4*n)),
              A033585_samples,
              "Triangular(4*n) A033585");
  check_equal(vector(#A033585_samples,t,t--; A033585(t)),
              A033585_samples,
              "A033585(t) vs A033585_samples");

  check_equal(vector(100,t,t--; A033585(t)),
              vector(100,t,t--; 2*t*(4*t+1)),
              "A033585(t) = 2*t*(4*t+1)");

  my(want=vector(10,t, A033585(t)));  \\ t>=1
  check_equal(select(n-> a(n)==0 && a(n+1)==0, [1..want[#want]]),
              want,
              "n of pair 0,0 = Triangular(4*n)");

}

\\ v=select(n->a(n)==0 && a(n+1)==0, [1..500])
\\ v=select(n->a(n-1)==0 && a(n)==0, [2..500])



\\-----------------------------------------------------------------------------
\\ contemplating a(n) with its own sqrtint

a_by_sqrtint(n) =
{
  n--;
  my(r,
     s = sqrtint(8*n+1,&r),
     h = s%2,
     d = (s-1)\2, k = n - d*(d+1)/2,
     t = s-1);
  s^2+r == 8*n+1 || error();
  3*d == 3*(s+(s%2))/2 - 3    || error();
  3*d == 3*(t-(t%2))/2    || error();
  k == n - (s-1)\2 * ((s-1)\2 + 1)/2 || error();
  3*d +k == 3*(s-2+(s%2))/2  + n - (s-2+(s%2))/2 * ((s-2+(s%2))/2 + 1)/2  || error();
  d == t\2 || error();
  t == 2*d+1-h || error();
  s == 2*d+2-h || error();
  k == (r + if(t%2,2*t+1))/8 || error();
  k == (r + if(h==0,2*s-1))/8 || error();
  k == (r + if(h==0,2*s-1))/8 || error();
  k == (r + if(h==0,2*(2*d+2-h)-1))/8 || error();
  k == (r + if(h==0,4*d+3))/8 || error();
  3*d == 3*(s+h)/2 - 3    || error();
  3*d+k == 3*(s+h)/2 - 3 + (r + if(h==0,2*s-1))/8   || error();
  3*d+k == 12*(s+h)/8 - 3 + (r + if(h==0,2*s-1))/8   || error();
  3*d+k == (12*s + r + if(h,12,2*s-1))/8 - 3   || error();
  p((3*d+k) % 12) == p(((12*s + r + if(h,12,2*s-1))/8 - 3) % 12) || error();
  p((3*d+k) % 12);
}

{
  check_equal(vector(#A317948_samples,n,a(n)),
              A317948_samples,
              "a() vs A317948_samples");
}




\\-----------------------------------------------------------------------------
print("end");