\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;
L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

check_exact(got,want) =
{
  if(got!= want,
     print("wrong\n"); quit(1),
     print("ok"));
}

\\-----------------------------------------------------------------------------

\\        *
\\        |
\\        |   L
\\        v
\\        *<-----*
\\               |
\\               |     L
\\               v
\\        *----->*
\\
\\           R

\\ Z + A*b^k*R(k) + B*b^(k+1)*R(k+1) + C*b^(k+2)*R(k+2) = gg(k)
{
my(k=3);
v = matsolve([ 1, L(k),   L(k+1), L(k+2);
               1, L(k+1), L(k+2), L(k+3);
               1, L(k+2), L(k+3), L(k+4);
               1, L(k+3), L(k+4), L(k+5) ],
         [ R(k); R(k+1); R(k+2); R(k+3) ]);
}
v

R_from_L(k) = -2 + 1/2*L(k+2);

vector(10,k,my(k=k-1); R_from_L(k))
vector(10,k,my(k=k-1); R(k))

check_equal(vector(100,k,my(k=k); R_from_L(k)), \
            vector(100,k,my(k=k); R(k)))


\\-----------------------------------------------------------------------------

\\ Z + A*b^k*R(k) + B*b^(k+1)*R(k+1) + C*b^(k+2)*R(k+2) = gg(k)
{
my(k=3);
v = matsolve([ 1, L(k),   L(k-1), L(k-2);
               1, L(k+1), L(k+0), L(k-1);
               1, L(k+2), L(k+1), L(k+0);
               1, L(k+3), L(k+2), L(k+1) ],
         [ R(k); R(k+1); R(k+2); R(k+3) ]);
}
v

R_from_L(k) = -2 + 1/2*L(k) + L(k-1) + L(k-2);

vector(10,k,my(k=k+2); R_from_L(k))
vector(10,k,my(k=k+2); R(k))

check_equal(vector(100,k,my(k=k+2); R_from_L(k)), \
            vector(100,k,my(k=k+2); R(k)))
