\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\ 1 1  1
\\ 2 2.88561808316413  0.961872694388042
\\ 3 6.57726068495955  0.939608669279935
\\ 4 14.3820623565768  0.958804157105117
\\ 5 29.8785066299894  0.963822794515786
\\ 6 60.8713951768146  0.966212621854201
\\ 7 122.857172270465  0.967379309216262
\\ 8 246.828726457766  0.967955790030455
\\ 9 494.771834832366  0.968242338223809
\\ 10 990.658051581571  0.968385192161848
\\ 11 1982.43048507998  0.968456514450406
\\ 12 3965.97535207702  0.968492149469358
\\ 13 7933.06508607023  0.968509960452964
\\ 14 15867.2445540593  0.968518864314184

\\-----------------------------------------------------------------------------
\\                       
\\   D---C              D--B
\\   | 1 |      
\\   A---BD---C 
\\        | 0 | 
\\        A---B 
\\
\\   D---C     D---C              D--AB        D,AB,CD,D
\\   | 3 |     | 1 |              AB--CD
\\   A---BD---CA---BD---C         CD--B
\\        | 2 |     | 0 | 
\\        A---B     A---B 
\\
\\ vertices D, AB, CD, B, 
\\ edges D-AB, CD-B, AB-CD, 
\\
\\            D---C     D---C              D--AB
\\            | 3 |     | 1 |              AB--ACD
\\            A---BD---CA---BD---C         ACD--B
\\                 | 2 |     | 0 |                 ACD--ABC  middle
\\  D---C     D---CA---B     A---B         ABC--D
\\  | 3 |     | 1 |                        ABC--CD
\\  A---BD---CA---BD---C                   CD--B
\\       | 2 |     | 0 |                   
\\       A---B     A---B        D,AB,ACD,B, D,ABC,CD,B
\\
\\ vertices D, AB, ACD, B, D, ABC, CD, B, 
\\ edges D-AB, ACD-B, D-ABC, CD-B,    evens
\\       AB-ACD, ABC-CD, B-D,         odds
\\
\\                      /d\ 
\\     D---C        1  Da cC   
\\     | 2 |            \b/    
\\     A---BD---C       /d\  /d\    
\\          | 1 |   0  Aa cBDa cC  1
\\          A---B       \b/  \b/    
\\                           /d\ 
\\                          Aa cB  0
\\                           \b/ 
\\   ABCD--ABCD
\\   => 01CA--A0C1   x2 vert
\\      A0B1--D1C0   across, AC connect
\\
\\                          /d\ 
\\        D---C            Da cC   
\\        | 2 |             \b/    
\\   D---CA---B        /d\  /d\    
\\   | 1 |            Da cCAa cB    D1C0--A0B1 across AC connect
\\   A---B             \b/  \b/    
\\                     /d\ 
\\                    Aa cB  
\\                     \b/ 

TA_vertex_to_edge(v) = \
  [ TA_transition(v,0), TA_transition(v,1), 0 ];
TA_edge_to_edge(e) =
{
  if(e[3],
    [ TA_transition(e[1],0), TA_transition(e[2],1), 1 ],
    [ TA_transition(e[1],1), TA_transition(e[2],0), 1 ]);
}

TA_vertex_name(v) =
{
  concat([if(v[1],"A",""),
          if(v[2],"B",""),
          if(v[3],"C",""),
          if(v[4],"D","")]);
}
TA_edge_name(e) = concat([TA_vertex_name(e[1]),"-",TA_vertex_name(e[2]), \
                         "(",e[3],")"]);
print(TA_edge_name(TA_edge_to_edge([ TA_AB, TA_CD, 0])));
print(TA_edge_name(TA_edge_to_edge([ TA_AB, TA_CD, 1])));


TA_edge_initial = [ TA_D, TA_B ];
edges_expand() =
{
  my(vertices_list = List([]));
  for(i=1,#vertices,
     for(bit=0,1,
        listput(vertices_list, TA_transition(vertices[i],bit))));

  my(edges_list = List([]));
  for(i=1,#vertices,
     listput(edges_list, TA_vertex_to_edge(vertices[i])));
  for(i=1,#edges,
     listput(edges_list, TA_edge_to_edge(edges[i])));
  vertices = Vec(vertices_list);
  edges = Vec(edges_list);
}
edges_ga() =
{
  my(total = 0);
  my(total_by_sqrt = vector(9));
  for(i=1,#edges, 
      my(e = edges[i],
         d1 = vecsum(e[1]),
         d2 = vecsum(e[2]),
         num = d1*d2,
         den = d1+d2);
     total += 2*sqrt(num) / den);
  total;
}
edges_ga_sqrts() =
{
  my(total_by_sqrt = vector(9));
  for(i=1,#edges, 
      my(e = edges[i],
         d1 = vecsum(e[1]),
         d2 = vecsum(e[2]),
         num = d1 * d2,
         den = d1 + d2);
     total_by_sqrt[num] += 2/den);

  for(i=1,#total_by_sqrt,
     if(total_by_sqrt[i],
        print("  sqrt("i") * "total_by_sqrt[i])));
}

edges_print() =
{
  print1("vertices ");
  for(i=1,#vertices, print1(TA_vertex_name(vertices[i]),", "));
  print();

  print1("edges ");
  for(i=1,#edges, print1(TA_edge_name(edges[i]),", "));
  print();
}
vertices = [ TA_0 ];
edges = [ ];
{
  for(k=0,999,
     my(ga = edges_ga());
     my(num_edges = #edges);
     my(f = if(#edges, ga / num_edges));
     print(k"  "num_edges"  "ga"  "f);
     edges_ga_sqrts();
     if(k<=4,
        edges_print();
        \\ print(vertices);
        \\ print(edges);
        print();
       );

     if(k==5,break());
     edges_expand();
  );
}

\\ 2 2.88561808316413  0.961872694388042
\\   2 4/3
\\   4 1/2
\\ 2  3  2.885618083164126731735584966  0.9618726943880422439118616552
\\   sqrt(2)  4/3
\\   sqrt(4)  1/2

\\ 3 6.57726068495955  0.939608669279935
\\   2 4/3
\\   3 1
\\   6 4/5
\\   9 1/3
\\ 3  7  6.577260684959546503820858567  0.9396086692799352148315512238
\\   sqrt(2)  4/3
\\   sqrt(3)  1
\\   sqrt(6)  4/5
\\   sqrt(9)  1/3

\\ 9 494.771834832366  0.968242338223809
\\   2 44
\\   3 32
\\   4 125/2
\\   6 384/5
\\   9 64/3
\\ 9  511  494.7718348323683334817040030  0.9682423382238127856784814149
\\   sqrt(2)  44
\\   sqrt(3)  32
\\   sqrt(4)  125/2
\\   sqrt(6)  384/5
\\   sqrt(9)  64/3

\\   sqrt(4)*125/2 + sqrt(9)*64/3 == 189
\\   sqrt(2)*44
\\   sqrt(3)*32
\\   sqrt(6)*384/5

\\ 14  16383  15867.24455405769874273072433  0.9685188643140876971696712645
\\   sqrt(2) * 4100/3
\\   sqrt(3) * 1024
\\   sqrt(4) * 4093/2
\\   sqrt(6) * 12288/5
\\   sqrt(9) * 2048/3

formula_terms(k) =
{
  print("formula k="k);
  print("  sqrt(2) * ", (2^(k-2)+4)/3);
  print("  sqrt(3) * ", 2^(k-4));
  print("  sqrt(4) * ", (2^(k-2) -3) / 2);
  print("  sqrt(6) * ", 3*2^(k-2) /5);
  print("  sqrt(9) * ", 2^(k-3) / 3);
}
formula(k) =
{
  sqrt(2) * (2^(k-2)+4)/3             \\ 1,1
  + sqrt(3) * 2^(k-4)                 \\ 1,3
  + sqrt(4) * (2^(k-2) -3) / 2        \\ 2,2
  + sqrt(6) * 3*2^(k-2) /5            \\ 2,3
  + sqrt(9) * 2^(k-3) / 3;            \\ 3,3
}
formula(k) =
{
  (3/8 + 1/12*sqrt(2) + 1/16*sqrt(3) + 3/20*sqrt(6))* 2^k
  -3 + 4/3*sqrt(2)
}
\\ (a*sqrt(2)+b)*(c*sqrt(3)+d) == (3/8 + 1/12*sqrt(2) + 1/16*sqrt(3) + 3/20*sqrt(6))
\\ a*c == 3/20
\\ c*b == 1/16
\\ a*d == 1/12
\\ b*d == 3/8
\\
\\ 1/12/d*c == 3/20
\\ c*b == 1/16
\\
\\ 1/12*1/16*20/3 == b*d  == 5/144
\\ 
formula_terms(5)

\\ connect  k=5   2--1 to 1--2
\\
ga_by_recurrence(k) =
{
  if (k<5, return(formula(k)));
  2*formula(k-1)
  - 2*  2*sqrt(2*1)/(2+1)
  + 3*  2*sqrt(2*2)/(2+2);
}


fraction(k) =
{
  3/8
  + sqrt(2)/12
  + sqrt(3)/16
  + sqrt(6) * 3/20
  + (
     - 21/8
     + 17/12 * sqrt(2)
     + 1/16 * sqrt(3)
     + 3/20 * sqrt(6)
    )
  / (2^k-1);

}
{
  for(k=5,10,print("k="k,"  ",
                  ga_by_recurrence(k),"  ",
                  formula(k),"  ",
                  fraction(k)));
}
my(k=50000); print(formula(k)/(2^k-1)); print(fraction(k));

{
  limit = 36/240 * (sqrt(3)+ 20/36) * (sqrt(2) + 15/36)
         + 49/144;
  print(limit);
  \\ 0.9685277670882894663085320
}
\\ sqrt(3)*(3/20*sqrt(2) + 1) + 11/24
\\ 
\\ {
\\   \\ largest root of
\\   nearly_equal(limit^4
\\                - 11/6*limit^3 
\\                - 12023/2400*limit^2
\\                + 463309/86400*limit
\\                + 1438086961/207360000
\\                ,
\\                0
\\               )
\\ }
