\\ Copyright 2015, 2017, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

join(v,sep=",") = if(#v==0,"", concat(vector(2*#v-1,i, if(i%2,Str(v[(i+1)/2]),sep))));
join([])=="" || error("join empty");
join([1])=="1" || error("join 1");
join([1,2])=="1,2" || error("join 1,2");
join([1,2]~)=="1,2" || error("join col");

to_binary(n) = subst(Pol(digits(n,2)),'x,10);    \\ n to decimal digits which are bits
from_binary(n) = from_binary_vector(digits(n));  \\ from decimal digits of n are bits
from_binary_vector(v) = subst(Pol(v),x,2);   \\ high to low

binary_string(n) =
{
 \\ print("n=",n);
  if(type(n)=="t_MAT",
     my(s=matsize(n));
     if(s[1]==1,n=n[1,],
        s[2]==1,n=n[,1],
        return(join(vector(s[1],i,binary_string(n[i,])),"\n"))));
  if(type(n)=="t_COL", n=Vec(n));
  if(type(n)=="t_VEC",
     \\ print("v=",join(apply(binary_string,n),", "));
     return(Str("[",join(apply(binary_string,n),", "),"]")));
  \\ print("n=",n);
  concat(apply((x)->Str(x),binary(n)));
}
binary_string(5)=="101" || error("binary 5");
binary_string([5])=="[101]" || error("binary [5]");
binary_string([5,9]~)=="[101, 1001]" || error("binary col");
binary_string([1;2])=="[1, 10]" || error("binary mat col");

b=I+1;

\\-----------------------------------------------------------------------------

dragon_points(k) = {
  my(v=[0,1]);
  for(i=1,k, my(end = (1+I)*v[#v]);
      v = concat(v, apply(c->(end - I*c), Vecrev(v[^-1]))));
  v;
}

\\  0 *---------> b^k              * b^2
\\      \     /                    |
\\       \   /                     3----2 b
\\        v v                           |
\\         *                       0----1 1
\\
\\ N=3   bpow=b^2 = 2*I
\\ then 2*I/b*-I == 1-I

n_to_xy(n) =
{
  my(v = binary(n),
     bpow = b^(#v-1),
     ret = 0);
  v = concat(0,v);
  for(i=2,#v,    \\ bits high to low
     if(v[i]==1 && v[i-1]==1, ret += I*bpow);
     if(v[i]==1 && v[i-1]==0, ret +=   bpow);
     if(v[i] != v[i-1], bpow *= I);
     bpow /= b);
  ret;
}

{
  for(k=0,5,
     check_exact(vector(2^k+1,n,my(n=n-1); n_to_xy(n)),
                 dragon_points(k),
                 Strprintf("k=%d",k)));
}

vector_modulo(v,i) = v[(i% #v)+1];
BlobN(k) = 1/5*(2^(k+1) - vector_modulo([12, 9, 3, 6],k));

\\
\\      -2+4*I
\\                   1+3*I                10
\\ -3+2*I
\\                        2+I              5
\\                 0
\\      -2-I
\\                             3-2*I      13
\\          -1-3*I    
\\                        2-4*I    <--    20
BlobLinkOffset(k) = \
  vector_modulo([ 2-4*I,  3-2*I,  2+I,  1+3*I,
                          -2+4*I, -3+2*I, -2-I, -1-3*I  ],k);
BlobNlinkpos(k) =
{
  -I/3 * (b^(k+1) + BlobLinkOffset(k));
}
check_exact(vector(100,k,my(k=k+3); BlobNlinkpos(k)), \
            vector(100,k,my(k=k+3); n_to_xy(BlobN(k))));


BlobOffset(k) = \
  vector_modulo([ -1+2*I, -3+I, -4-2*I,  -2-3*I, \
                   1-2*I,  3-I,  4+2*I,   2+3*I  ],k);
\\                  5       10    20       13
BlobNpos(k) = - 1/3 * I * (b^(k+1) + BlobOffset(k));

{
  for(k=4,24,
     my(n=BlobN(k)+3, z=n_to_xy(n), fz=BlobNpos(k));
     printf("%2d %4d %-14s   %3d %3d  %.2f %.2f\n",
            k, n, binary_string(n),
            real(z), imag(z),
            real(fz), imag(fz)));
}
check_exact(vector(100,k,my(k=k+3); BlobNpos(k)), \
            vector(100,k,my(k=k+3); n_to_xy(BlobN(k)+3)));


\\ link step
\\ [-3 + 6*I, -6 + 3*I, -6 - 3*I, -3 - 6*I,
\\   3 - 6*I,  6 - 3*I,  6 + 3*I,  3 + 6*I]

\\ [ 3 + 6*I, -3 + 6*I, -6 + 3*I, -6 - 3*I,
\\  -3 - 6*I,  3 - 6*I,  6 - 3*I,  6 + 3*I ]

\\ 0 + 1 + ... + k-2         + k-1 + k   +        k-2 + ... + 1 + 0
\\ 0 + 1 + ... + k-2 + k-1   + k   + k+1 +  k-1 + k-2 + ... + 1 + 0
\\
\\ JA(k) = A(k+1) - A(k) = Blob(k+1) + Blob(k-1)
