\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;
L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
RQ(k)=1 + sum(j=0,k-1, LQ(j));
RQ = memoize(RQ);
B(k) = R(k+1);
A(k) = 2^(k-1) - B(k)/4;
JA(k) = A(k+1) - 2*A(k);
JEQ(k) = (3/5 + 1/5*I)*b^k - (1/10 - 3/10*I)*(-1)^k;
LQ(k) = if(k<0,error("LQ() is for k>=0")); \
        if(k==0,return(1)); \
        if(k<=3,return(2^(k-1))); \
        LQ(k-1) + 2*LQ(k-3);
LQ = memoize(LQ);
gLQ(x) = (1 + x^2) / (1 - x - 2*x^3);

GLQ(k) = 1/2*b^max(1,k);
GLQN(k) = LQ(k)*1/2*b^max(1,k);

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_count = 0;
check_equal(got,want) =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok "check_count));
}

check_exact(got,want) =
{
  check_count++;
  if(got!= want,
     print("wrong\n");
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

\\-----------------------------------------------------------------------------

GRQN(k) = \
  if(k==0,return(1/2-1/2*I)); \
  (57-21*I)/82 \
  + b^k/82 * ( \
      ( 70 - 15*I) * RQ(k) \
    + (  3 + 14*I) * RQ(k+1) \
    + (-17 - 11*I) * RQ(k+2) \
    );


vector(10,k, GRQN(k) - (b*GRQN(k-1) + 2*b^3*GRQN(k-3) + 3/2-9/2*I))

check_exact(vector(100,k,my(k=k+3); GRQN(k)), \
            vector(100,k,my(k=k+3); b*GRQN(k-1) + 2*b^3*GRQN(k-3) + 3/2 - 9/2*I));
