\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("../tools/memoize.gp");
read("pol-pfrac.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));
gf_term(g,n) = my(v=gf_terms(g,n+1)); v[#v];

L(k) = if(k<0,error("L() is for k>=0")); \
        if(k<=3,return(2^k)); \
        L(k-1) + 2*L(k-3);
L=memoize(L);
gL(x) = -1 + 2* (1 + x^2) / (1 - x - 2*x^3);

R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
gR(x)= 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

A(k) = 2^(k-1) - 1/4 * B(k)
D(k) = A(k);
B(k) = if(k<0,error("B() is for k>=0")); R(k+1);
S(k) = B(k)/2 + 1;

\\-----------------------------------------------------------------------------
vector_non_zeros(v) = select((x)->x!=0, v);

str_poly_ascending__join_strings(v,sep) =
{
  concat(vector(2*#v-1,i, if(i%2,v[(i+1)\2],sep)));
}

str_poly_ascending(p) =
{
  if(type(p) == "t_VEC",
     return(str_poly_ascending__join_strings(apply(str_poly_ascending,p),
                                             "\n + ")));

  my(d = denominator(p));
  if(poldegree(d) != 0,
     p = numerator(p);
     my(low = polcoeff(d, 0));
     if(low==0, error("Oops, constant term of denominator is 0"));
     d /= low;
     p /= low;
     my(fra=lcm(vector_non_zeros(apply(denominator,Vec(p)))));
     if(fra==0, error("oops fra=",fra));
     p *= fra;
     my(p_single = (poldegree(p) == 0
                    || poldegree(p) == valuation(p,variable(p))));
     my(d_single = (poldegree(d) == 0
                    || poldegree(d) == valuation(d,variable(d))));
     p_single = 0;
     d_single = 0;
     return(concat([if(fra==1, "", Str(1/fra,"*")),
                    if(p_single,"","("),
                    str_poly_ascending(p),
                    if(p_single,"",")"),
                    "/",
                    if(d_single,"","("),
                    str_poly_ascending(d),
                    if(d_single,"",")")
                   ])));
  if(poldegree(p) <= 0, return(Str(p)));
  my(str="");
  for(i=0,poldegree(p),
     my(coeff=polcoeff(p,i));
     if(coeff == 0, next());
     if(str!="",
        if(real(coeff)<0 || (real(coeff)==0 && imag(coeff)<0),
           str=concat(str," - "); coeff = -coeff,
           str=concat(str," + ")));
     str = concat([str,coeff*variable(p)^i]));
  str;
}

\\ str = str_poly_ascending( (x^2 + 1/2*x + 1)/(2*x^3 + x - 1) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( ((-1/4 + 1/4*I)*x^2 + (1/8 + 1/8*I))/(x^3 - 1/4*I*x + (1/8 + 1/8*I)) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( (x^2)/(2*x^3 + x - 1) );
\\ print("str = "str);

\\-----------------------------------------------------------------------------

\\ D[k] > S[k]
gD(x) =    (1/2)/(1-2*x) +  (1/2)/(1-x) - 1/2*(2 + x + 2*x^2)/(1 - x - 2*x^3);
gS(x) =                                       (2 + x + 2*x^2)/(1 - x - 2*x^3);

\\ gf_terms(gDsubS(x),16)

\\ g(x) = (gDsubS(x) + ( 2 +3*x +5*x^2 +9*x^3 +14*x^4 +21*x^5 +32*x^6 +45*x^7 +56*x^8 +57*x^9 +20*x^10 ))/x^11;
\\ gf_terms(g(x),16)
\\ 
\\ \\ pol_partial_fractions(g(x))
\\ 
\\ {
\\   check_exact(g(x),
\\               (1/2)/(1 - x)
\\               + 1024/(1 - 2*x)
\\               - (1803/2 + 627*x + 1065*x^2)/(1 - x - 2*x^3))
\\ }
\\ 
\\ g = g(x);
\\ {
\\   for(i=0,5,
\\      my(t = gf_term(g,0));
\\      g = (g - t)/x;
\\      print(str_poly_ascending(pol_partial_fractions(g)));
\\   );
\\ }

\\-----------------------------------------------------------------------------
\\ D[k+1] < 2*D[k]

\\ S(k) = S(k-1) + 2*S(k-3)
\\ if S(k) < 2*S(k-1)
\\ S(k-1) + 2*S(k-3) < S(k-1)  + 2*1/4*S(k-1) 
\\                   = 3/2*S(k-1)

gDsubS(x) = (1/2)*1/(1-x) \
          + (1/2)*1/(1-2*x) \
          - 3/2*(2 + x + 2*x^2)/(1 - x - 2*x^3);
check_exact(gD(x)-gS(x), gDsubS(x));
print(gf_terms((2 + x + 2*x^2)/(1 - x - 2*x^3), 10));
\\ 2, 3, 5, 9, 15, 25, 43, 73, 123, 209 = A003476 = RQ = BQ = S

DsubS_ref(k) = D(k) - S(k);
print(vector(16,k,my(k=k-1); DsubS_ref(k)))

DsubS(k) = (2^k+1 - S(k))/2 - S(k);
check_exact(vector(64,k,my(k=k-1); DsubS(k)) \
            - vector(64,k,my(k=k-1); DsubS_ref(k)), \
            vector(64,k,my(k=k-1); 0));

DsubS(k) = 1/2*(1 + 2^k - 3*S(k));
check_exact(vector(64,k,my(k=k-1); DsubS(k)) \
            - vector(64,k,my(k=k-1); DsubS_ref(k)), \
            vector(64,k,my(k=k-1); 0));

\\ DsubS(k) > 0
\\ 1 + 2^k - 3*S(k) > 0
\\ 2^k > 3*S(k)
