#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use lib "$FindBin::Bin/../tools";
use MyDragon 'transform_to_horizontal','xy_rotate_plus90','xy_rotate_minus90',
  'BlobN','BlobNend','BP1_xy','BP6_xy';
use MyPrintwrap ':all';
use MyLineSegments;

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 10;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyLevelSmaller}{",$level-2,"}\n";
my $path = Math::PlanePath::DragonCurve->new;

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  return ($x, $y);
}
sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_right_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_n_is_left_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_left_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $BlobN = BlobN($level);
my $BlobNend = BlobNend($level);
my $BlobN_small = BlobN($level-2);
my $BlobNend_small = BlobNend($level-2);
my ($bp1_x,$bp1_y) = BP1_xy($level);
my ($bp6_x,$bp6_y) = BP6_xy($level);
my ($bp1_small_x,$bp1_small_y) = BP1_xy($level-2);
my ($bp6_small_x,$bp6_small_y) = BP6_xy($level-2);

sub transform_blob_first {
  my ($x,$y) = @_;
  $x -= $bp1_small_x;
  $y -= $bp1_small_y;
  ($x,$y) = xy_rotate_minus90($x,$y);
  $x += $bp6_x;
  $y += $bp6_y;
  return ($x,$y);
}
sub transform_blob_second {
  my ($x,$y) = @_;
  $x -= $bp6_small_x;
  $y -= $bp6_small_y;
  ($x,$y) = xy_rotate_minus90($x,$y);
  $x += $bp1_x;
  $y += $bp1_y;
  return ($x,$y);
}

$Printwrap = 2;
{
  my ($x,$y) = $path->n_to_xy($BlobN);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BlobStart) at ($x,$y) [my small dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($BlobNend);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BlobEnd) at ($x,$y) [my small dot];\n");
}
{
  my ($x,$y) = ($bp6_x,$bp6_y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BP6) at ($x,$y) [my dot];\n");
}
{
  my ($x,$y) = ($bp1_x,$bp1_y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BP1) at ($x,$y) [my dot];\n");
}

{
  my ($x,$y) = $path->n_to_xy($BlobN_small);
  ($x,$y) = transform_blob_first($x,$y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BlobStart-a) at ($x,$y) [my small dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($BlobNend_small);
  ($x,$y) = transform_blob_first($x,$y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BlobEnd-a) at ($x,$y) [my small dot];\n");
}
{
  my ($x,$y) = ($bp6_small_x,$bp6_small_y);
  ($x,$y) = transform_blob_first($x,$y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BP6a) at ($x,$y) [my dot];\n");
}

{
  my ($x,$y) = $path->n_to_xy($BlobN_small);
  ($x,$y) = transform_blob_second($x,$y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BlobStart-b) at ($x,$y) [my small dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($BlobNend_small);
  ($x,$y) = transform_blob_second($x,$y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BlobEnd-b) at ($x,$y) [my small dot];\n");
}
{
  my ($x,$y) = ($bp1_small_x,$bp1_small_y);
  ($x,$y) = transform_blob_second($x,$y);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (BP1b) at ($x,$y) [my dot];\n");
}

my $curve_segments = MyLineSegments->new;
my $blob_segments = MyLineSegments->new;
my $both_segments = MyLineSegments->new;

foreach my $n ($BlobN_small .. $BlobNend_small-1) {
  my $is_left = path_n_is_left_boundary($path,$n, $n_hi);
  my $is_right = path_n_is_right_boundary($path,$n, $n_hi);
  next unless $is_left || $is_right;

  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  {
    my ($x1,$y1) = transform_blob_first($x1,$y1);
    my ($x2,$y2) = transform_blob_first($x2,$y2);
    $blob_segments->add($x1,$y1, $x2,$y2);
  }
  {
    my ($x1,$y1) = transform_blob_second($x1,$y1);
    my ($x2,$y2) = transform_blob_second($x2,$y2);
    $blob_segments->add($x1,$y1, $x2,$y2);
  }
}
foreach my $n ($BlobN .. $BlobNend-1) {
  my $is_left = path_n_is_left_boundary($path,$n, $n_hi);
  my $is_right = path_n_is_right_boundary($path,$n, $n_hi);
  next unless $is_left || $is_right;

  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  if ($blob_segments->is_traversed($x1,$y1, $x2,$y2)) {
    $both_segments->add($x1,$y1, $x2,$y2);
    $blob_segments->remove($x1,$y1, $x2,$y2);
  } else {
    $curve_segments->add($x1,$y1, $x2,$y2);
  }
}

$Printwrap = 2;
printwrap("\\draw[my curve colour]");
$Printwrap += 2;
$curve_segments->printwrap (transform => \&transform);
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my blob colour]");
$Printwrap += 2;
$blob_segments->printwrap (transform => \&transform);
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw[my both colour]");
$Printwrap += 2;
$both_segments->printwrap (transform => \&transform);
printwrap(";\n");

print "  % end generated\n";
exit 0;
