#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon 'chamfer';
use MyPrintwrap ':all';


my $level = 9;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

sub is_touch {
  my ($n, $n_hi) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_other ($path->xy_to_n_list($x,$y)) {
    next if $n_other == $n;
    next if $n_other > $n_hi;
    return 1;
  }
  return 0;
}

sub level_any_touch {
  my ($level) = @_;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  foreach my $n ($n_lo .. $n_hi) {
    if (is_touch($n, $n_hi)) {
      return 1;
    }
  }
  return 0;
}

$Printwrap = 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 4;
printwrap("\\node (start-$level) at (0,0) [my dot];\n");
printwrap("\\node (end-$level) at ($x_hi,$y_hi) [my dot];\n");
printwrap("\\draw (start-$level)");
my $any_touch = level_any_touch($level);
foreach my $n ($n_lo+1 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  if ($any_touch) {
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    my ($tdx,$tdy) = $path->n_to_dxdy($n);
    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($tdx);
    my $ty = $y . chamfer($tdy);
    printwrap("--($sx,$sy)");
    printwrap("--($tx,$ty)");
  } else {
    printwrap("--($x,$y)");
  }
}
printwrap("--(end-$level);\n");

print "  % end generated\n";
exit 0;
