\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------
\\ Ixy = real*imag
\\ Ix = Iz(k-1) + imag^2
\\ Iy = Iz(k-1) + real^2
\\ Ix-Iy = imag^2 - real^2

Ix(k) =
{
  3/50 * 4^k
  - 1/200 * vector_modulo([29, 23, 21, 27 ],k) * 2^k
  - vector_modulo([1/25, 1/100],k)
  + if(k==0, + 1/8)
  ;
}
Iy(k) =
{
  7/50 * 4^k
  - 1/200 * vector_modulo([1, 27, 49, 23],k) * 2^k
  - vector_modulo([1/100, 1/25],k)
  + if(k==0, - 1/8)
  ;
}
Ixy(k) =
{
  -3/50 * 4^k
  + vector_modulo([1/50, 7/50, -1/50, -7/50],k) * 2^k
  + vector_modulo([1/25, -1/25, 1/25, -1/25],k)
  ;
}

sG(k) = (3/5 + 1/5*I) * 2^k + (2/5 - 1/5*I) * (-I)^k;

\\-----------------------------------------------------------------------------

{
  my(count=0);
  for(k=2,100,
     my(x = 4* (Ix(k) - Iy(k)),
        y = 4* Ixy(k),
        g = gcd(x,y));
     if(g!=1,
        print("k="k" gcd="g);
        count++);
     );
  print("count ",count," with common factor");
}

{
  my(count=0);
  for(k=2,100,
     my(x = 4* (Ix(k) - Iy(k)),
        y = 4* Ixy(k),
        Csquared = x^2 + y^2);
     if(! issquare(Csquared),
        print("k="k" non-square "C);
        count++);
     );
  print("count ",count," non-squares");
}

two_alpha_slope(k) =
{
  my(x = -4* (Ix(k) - Iy(k)),
     y = -4* Ixy(k),
     g = gcd(x,y));
  \\ print("x=",x," y=",y," gcd=",g);
  if(g!=0,
     x /= g; y /= g);
  x + y*I;
}
print("two_alpha_slope");
for(k=0,6,print("k="k"  ",two_alpha_slope(k)));


\\ p = sqrt( (C + A)/2 )
\\ q = sqrt( (C - A)/2 )

alpha_slope_ref(k) = sqrt(two_alpha_slope(k));

\\ exact root if possible
sqrt_exact(n) = if(issquare(n, &root), root, sqrt(n));

sqrt_by_half(z) =
{
  sqrt_exact( (sqrt_exact(real(z)^2 + imag(z)^2) + real(z)) / 2 )
  + sqrt_exact( (sqrt_exact(real(z)^2 + imag(z)^2) - real(z)) / 2 ) * I;
}
alpha_slope(k) = sqrt_by_half(two_alpha_slope(k));
print("alpha_slope");
for(k=0,16,print("k="k"  ",alpha_slope(k)));

check_exact(2*(4+3*I), (3 + I)^2);
check_equal(sqrt_by_half(2*(4+3*I)), 3 + I);

check_equal(vector(50,k,my(k=k+1); alpha_slope(k)), \
            vector(50,k,my(k=k+1); alpha_slope_ref(k))) /* k>=2 */

\\ real A016029 a(1) = a(2) = 1, a(2n + 1) = 2*a(2n) and a(2n) = 2*a(2n - 1) + (-1)^n.
\\   + 1/5*(3)/(1 - 2*x)
\\   + (2/5 - 1/5*x)/(1 + x^2)
\\   3/5*2^k + vector_modulo([2/5, -1/5, -2/5, 1/5],k)
\\ imag A007910 1/((1-2*x)*(1+x^2))
\\  = 1, 2, 3, 6, 13, 26, 51, 102, 205, 410, 819, 1638, 3277, 6554, 13107,
\\  = binary 1100,1100,1100,...
\\              = (4/5)/(1 - 2*x)
\\                + (1/5 + 2/5*x)/(1 + x^2)
\\      4/5*2^k + vector_modulo([1/5, 2/5, -1/5, -2/5],k)
\\ cf here 0,0,1,2,3,6,13,26,51,102,205,410,819,1638,3277,6554,13107,
\\
for(k=0,16,print1(imag(alpha_slope(k)),",")); print();
\\ for(k=4,16,print(factor(real(alpha_slope(k))), "  ", factor(imag(alpha_slope(k)))));

\\ GCD  4/5*2^k + vector_modulo([1/5, 2/5, -1/5, -2/5],k)
\\      3/5*2^k + vector_modulo([2/5, -1/5, -2/5, 1/5],k)
\\
\\      1/5*2^k + vector_modulo([-1/5, 3/5, 1/5, -3/5],k)
\\      3/5*2^k + vector_modulo([2/5, -1/5, -2/5, 1/5],k)
\\
\\      1/5*2^k + vector_modulo([-1/5, 3/5, 1/5, -3/5],k)  /* always odd */
\\                vector_modulo([1, -2, -1, 2],k)
\\
\\ 3*[1/5, 2/5, -1/5, -2/5] - 4*[2/5, -1/5, -2/5, 1/5] == [-1, 2, 1, -2]
\\ vector(16,k, (4*2^k + vector_modulo([1,2,-1,-2],k)) % 10) /* 0,5,0,5 */
\\ vector(16,k, (3*2^k + vector_modulo([2,-1,-2,1],k)) % 10) /* 5,0,5,0 */
\\ vector(16,k, (2^k + vector_modulo([-1, 3, 1, -3],k)) % 10) /* == 5 always */
\\ 2^k % 10 = [2,4,8,6] /* k>=1 */

GS_rotated(k) = GS(k) * (conj(b))^k;
print("GS_rotated");
for(k=0,6,print("k="k"  ",GS_rotated(k)));

I_minimum_on_end_line_ref(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  g - imag(g)/imag(s) * s;
}

print("to end line");
{
  for(k=0,26,
     my(l = I_minimum_on_end_line_ref(k));
     my(t = 1);
     my(diff = t - l);
     print("k="k"  ",l," target ",t," diff ",diff);
  );
}

GSF_simplified(k) = 2/5-1/5*I + (1/10+1/5*I)*(-I/2)^k;
sG_real(k) = 3/5*2^k + 1/5*vector_modulo([2, -1, -2, 1],k);
sG_imag(k) = 1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k);

GSF_real(k) =
{
  2/5 + 1/10*vector_modulo([1,2,-1,-2],k) *2^-k;
}
check_equal(vector(100,k,my(k=k-1); GSF_real(k) - real(GSF(k))), \
            vector(100,k,my(k=k-1); 0));

GSF_imag(k) =
{
  -1/5 + 1/10*vector_modulo([2,-1,-2,1],k) *2^-k;
}
vector(16,k,my(k=k-1); GSF_imag(k) - imag(GSF(k)))
check_equal(vector(100,k,my(k=k-1); GSF_imag(k) - imag(GSF(k))), \
            vector(100,k,my(k=k-1); 0));


I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  2/5 + 1/10*vector_modulo([1,2,-1,-2],k) *2^-k
  - (-1/5 + 1/10*vector_modulo([2,-1,-2,1],k) *2^-k)
  / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
  * (3/5*2^k + 1/5*vector_modulo([2, -1, -2, 1],k));
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  2/5
  + 1/10*vector_modulo([1,2,-1,-2],k) *2^-k
  - 1/2 * 2^-k
  * (-2/5*2^k + 1/5*vector_modulo([2,-1,-2,1],k))
  * (
     3 +
   (1/5*vector_modulo([2, -1, -2, 1],k) - 3*1/5*vector_modulo([-1, -2, 1, 2],k))
   / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
)
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  2/5
  + 1/10*vector_modulo([1,2,-1,-2],k) *2^-k
  - 1/2 * 2^-k
  * (-2/5*2^k + 1/5*vector_modulo([2,-1,-2,1],k))
  * (
     3 +
     1/5*vector_modulo([5, 5, -5, -5],k)
     / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
)
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  1
  + vector_modulo([-1, 1, 1, -1], k) * 2^-(k+1)

  +
  (- 1/2
   * 1/5*vector_modulo([5, 5, -5, -5],k)
   * 2^-k
   * -2/5*2^k

   - 1/2
   * 1/5*vector_modulo([5, 5, -5, -5],k)
   * 2^-k
   * 1/5*vector_modulo([2,-1,-2,1],k)
  )
  / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  1
  + 1/2*vector_modulo([-1, 1, 1, -1], k) / 2^k

  + 1/5 * vector_modulo([1,1,-1,-1],k)
  / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))

  - 1/10 * vector_modulo([2,-1,2,-1],k)
  / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
  / 2^k
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */


I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));
  (
   (2/5 + 1/10*vector_modulo([1,2,-1,-2],k) *2^-k)
   * (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))

   - (-1/5 + 1/10*vector_modulo([2,-1,-2,1],k) *2^-k)
   * (3/5*2^k + 1/5*vector_modulo([2, -1, -2, 1],k))
  )
  / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));

  1 -
  2^-k
  * (
     (3/5*2^k - 1/10*vector_modulo([1,2,-1,-2],k))
     * (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
     
     - (3/5*2^k + 1/5*vector_modulo([2, -1, -2, 1],k))
     * (1/5*2^k - 1/10*vector_modulo([2,-1,-2,1],k))
     
    )
  / (1/5*2^k + 1/5*vector_modulo([-1, -2, 1, 2],k))
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

I_minimum_on_end_line(k) =
{
  my(g = GSF(k));
  my(s = sG(k));
  if(imag(s)==0,return(g));

  1 +
  1/10 * (
   vector_modulo([1, 3, -1, -3],k)
   - 2^-k
  )
  / sG_imag(k)
  ;
}
check_equal(vector(200,k,my(k=k+1); I_minimum_on_end_line(k)), \
            vector(200,k,my(k=k+1); I_minimum_on_end_line_ref(k))); /* k>=2 */

