\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");
read("pol-pfrac.gp");

default(recover,0);
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

vector_modulo(v,i) = v[(i% #v)+1];
BlobDeltaLen(k) = if(k<4,0, 1/3*(2^ceil(k/2) - vector_modulo(BlobDeltaLen_vector,k)));
BlobDeltaLen_vector = [4, 5, 5, 7 ];
TotalNonBlob(k) = if(k<#TotalNonBlob_initial, TotalNonBlob_initial[k+1], 6*k-13);
TotalNonBlob_initial = [1, 2, 4, 8, 12];

BlobList(k) = {
  if(k<0,error("BlobList() is for k>=0")); \
  my(i=0,ret=List([]));
  while(i<=k, listput(ret,i);i++);
  i -= 3;  /* from k+1 down to k-2 */
  while(i>=0, listput(ret,i);i--);
  Vec(ret);
}

BlobStart(k) = 1/3*(conj(b)*b^k + boff(k));
BlobEnd(k) = 1/3*(2*b^k - I* boff(k-1));
boff_vector_1 = {[ 2 +  I, 1 + 3*I, -2 + 4*I, -3 + 2*I ]};
boff_vector_2 = {[ -2 -  I, -1 - 3*I, 2 - 4*I, 3 - 2*I ]};
boff_vector = concat(boff_vector_1, boff_vector_2);
boff(m) = vector_modulo(boff_vector,m);
b=1+I;
sqrt2 = quadgen(4*2);

\\-----------------------------------------------------------------------------

diag(z1, z2) = {
  my(delta = z2 - z1);
  if(real(delta) == 0 || imag(delta) == 0, abs(delta),
     abs(real(delta)) == abs(imag(delta)), sqrt2*abs(real(delta)),
     error("Oops, not diagonal: ",z1," to ",z2," delta ",delta));
}

move_start() = move_position = 0; move_total = 0;
move(z) = {
  move_total += diag(move_position,z);
  move_position = z;
}

DiagLen(k) = {
  my(debug=0,
     end=b^k);
  if(debug, print("k=",k," end=",end));
  move_start();
  move(1+I);
  move(2*I);
  for(j=5,k,
      if(j%2, move(BlobStart(j));
              if(debug, print("j=",j"  odd start of j=",j));
              ,
              move(BlobEnd(j-1)); 
              if(debug, print("j=",j"  even at end of j-1=",j-1));
              );
     );

 if (k%2 == 0,
     \\ across k=10
     \\ my(p = BlobEnd(k),
     \\    offset = I*I^(k/2));
     \\ if(debug, print("middle k=",k," even across to ",p,"  offset ",offset));
     \\ move(p + offset)

     \\ across k=8
     my(p = end + (-I)*BlobStart(k-2),
        offset = (1+I)*I^(k/2));
     if(debug, print("even across k-2 = ",k-2,"  to ",p,"  offset ",offset));
     move(p + offset);
     ,
     \\ across k=9
     my(p = BlobEnd(k),
        offset = (-1+I)*I^((k-1)/2));
     if(debug, print("middle k=",k," odd across to ",p," then offset ",offset));
     move(p);
     move(p+offset));

 if(debug, print("move_position ",move_position));

  forstep(j=k-2,5,-1,
      if(j%2, if(debug, print("tail j=",j,"  odd"));
              move(end + (-I)*BlobStart(j)),
              if(debug, print("tail j=",j,"  even"));
              move(end + (-I)*BlobEnd(j-1)));
     );
  
  move(end + 2);
  move(end + 1-I);
  move(end);
  if(move_position != end,
     print("k=",k," not end, got ",move_position));
  move_total;
}
\\ DiagLen(10);
\\ quit

v = vector(246,k,my(k=k+5); DiagLen(k));
print("DiagLen ", v);
print("  int    ", real(v));
print("  sqrt2  ", imag(v));

v = vector(#v,k,my(k=k+5); v[k-5] / ( (1 + sqrt2)*sqrt(2)^k));
print("  abs    ", v);

\\ 1 + sqrt(2) = 2.41421

\\ s = sum 1/sqrt2^i
\\ s = 1 + 1/sqrt2*s
\\ s = 1 / (1-1/sqrt2)
\\ s = 2 + sqrt2
\\ s + 1/2*s = 3 + 3/2*sqrt2
\\ s - sqrt2 - 1/sqrt2

quit

\\-----------------------------------------------------------------------------

BlobPathLenCumulative_by_sum(k) = sum(j=1,k, BlobPathLen(j));

BlobPathLenCumulative(k) = \
  sum(j=4,k, 1/3*(2*2^floor(j/2) - vector_modulo([8,5,10,7],j)));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));
vector(16,k,my(k=k+7); BlobPathLenCumulative(2*k+1))

BlobPathLenCumulative(k) = \
  1/3 * ( 2*sum(j=4,k, 2^floor(j/2)) \
          - sum(j=4,k, vector_modulo([8,5,10,7],j)) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  2/3 * sum(j=4,k, 2^floor(j/2)) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -4 \
  + 2/3 * ( sum(j=0,k, 2^floor(j/2)) ) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -4 \
  + 2/3 * ( 2^(floor(k/2)+1)-1  + 2^(floor(k/2)+(k%2))-1 ) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -4 - 4/3 \
  + 2/3 * ( 2*2^floor(k/2)  + 2^floor(k/2)*vector_modulo([1,2],k) ) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -16/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * (-30 + sum(j=0,k, vector_modulo([8,5,10,7],j)) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( sum(j=0,k, vector_modulo([8,5,10,7],j)) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 8*(floor(k/4)+1) \
          + 5*(floor((k-1)/4)+1) \
          + 10*(floor((k-2)/4)+1) \
          + 7*(floor((k-3)/4)+1) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 8*floor(k/4) + 8 \
          + 5*floor((k-1)/4) + 5 \
          + 10*floor((k-2)/4) + 10 \
          + 7*floor((k-3)/4) +7 );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 - 10 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 8*floor(k/4)  \
          + 5*floor((k-1)/4)  \
          + 10*floor((k-2)/4)  \
          + 7*floor((k-3)/4)  );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 - 10 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 30*floor(k/4)  \
            - vector_modulo([5,0,0,0],k)  \
            - vector_modulo([10,10,0,0],k)  \
            - vector_modulo([7,7,7,0],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 - 10 \
  + 1/3 * ( 2^floor(k/2)*vector_modulo([6,8],k) ) \
  + 1/3 * ( -30*floor(k/4)  \
            + vector_modulo([22,17,7,0],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  1/3 * ( 2^floor(k/2)*vector_modulo([6,8],k)   \
          - 30/4*k  \
          + vector_modulo([6,1+30/4,-9+2*30/4,-16+3*30/4],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)   \
          - 30/2*k  \
          + vector_modulo([12, 17, 12, 13],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = {
  if(k<4,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   ));
}
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

vector(16,k,my(k=k+3); BlobPathLenCumulative(k))

\\-----------------------------------------------------------------------------

PathLen_by_blobs(k) = \
  BlobPathLenCumulative(k) + BlobPathLenCumulative(k-2) + TotalNonBlob(k);
vector(16,k,my(k=k+7); PathLen_by_blobs(k))

PathLen(k) = {
  if(k<4,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   ))
  +
  if((k-2)<4,0,
     1/6 * ( 2^floor((k-2)/2)*vector_modulo([12,16],(k-2))
             - 15*(k-2)
             + vector_modulo([12, 17, 12, 13],(k-2))   ))
  + TotalNonBlob(k);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

PathLen(k) = {
  if(k<4,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   ))
  +
  if(k<6,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([6,8],k)
             - 15*k
             + vector_modulo([42, 43, 42, 47],k)   ))
  + 6*k-13
  + if(k<5, [1-(6*0-13), 2-(6*1-13), 4-(6*2-13), 8-(6*3-13), 12-(6*4-13)][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

PathLen(k) = {
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   )
  +
  if(k<6,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([6,8],k)
             - 15*k
             + vector_modulo([42, 43, 42, 47],k)   ))
  + 6*k
  - 13
  + if(k<5, [10, 6, 4, 3, 1][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

PathLen(k) = {
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   )
  +
     1/6 * ( 2^floor(k/2)*vector_modulo([6,8],k)
             - 15*k
             + vector_modulo([42, 43, 42, 47],k)   )
  + 6*k
  - 13
  + if(k<5, [2, 0, 0, 0, 0][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

PathLen(k) = {
  1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
          + 2^floor(k/2)*vector_modulo([6,8],k)
          + vector_modulo([54, 60, 54, 60],k)   )
  + k
  - 13
  + if(k<5, [2, 0, 0, 0, 0][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

PathLen(k) = {
  2^floor(k/2)*vector_modulo([3, 4],k)
  + vector_modulo([0,1],k)
  + k - 4
  + if(k<5, [2, 0, 0, 0, 0][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

PathLen(k) = {
  if(k==0,   1,
     k%2==0, 3*2^floor(k/2) + k - 4,
             4*2^floor(k/2) + k - 3);
}
check_exact(vector(64,k,my(k=k-1); PathLen(k)), \
            vector(64,k,my(k=k-1); PathLen_by_blobs(k)));

print("PathLen ", vector(16,k,my(k=k-1); PathLen(k)))



\\ Path(k) = k*2^(k-4) + vector_modulo([0],k);
\\ vector(16,k,my(k=k+3); Path(k) - Path_by_blobs(k))
