// Copyright 2015 Kevin Ryde
//
// This file is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 3, or (at your option) any later
// version.
//
// This file is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this file.  See the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.


level = 6;
linewidth = .1;
$fn = 20;

module dragon(level, x1,y1, x2,y2) {
    if (level == 0) {
        dx = x2 - x1;
        dy = y2 - y1;
        multmatrix([ [ dx,-dy, 0, x1 ],
                     [ dy, dx, 0, y1 ],
                     [  0,  0, 1, 0  ],
                     [  0,  0, 0, 1  ] ])
        cube([1, linewidth, linewidth]);
    } else {
        // Recurse as two curves drawn from the ends towards the middle.
        // The middle is half length along and half length perpendicular.
        //
        //   x1,y1              x2,y2
        //      --_          _--
        //          -      -
        //            mx,my
        //
        mx = (x1 + x2 - y1 + y2)/2;
        my = (x1 - x2 + y1 + y2)/2;
        dragon(level-1, x1,y1, mx,my);
        dragon(level-1, x2,y2, mx,my);
   }
}


sphere(2*linewidth);  // highlight the start of the curve
dragon(level, 0,0, pow(2,level/2),0);


// Local variables:
// compile-command: "openscad --render --imgsize=1000,800 -o /tmp/dragon-cube.png dragon-cube.scad && xzgv /tmp/dragon-cube.png"
// End: