#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','dir','DirMaxN','chamfer';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;
my $dirmax_start_level = 1;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my %maxes;
foreach my $k ($dirmax_start_level .. $level) {
  my $n = DirMaxN($k);
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $maxes{"$x,$y,$x2,$y2"} = 1;
}

{
  my $n_lo = 0;
  my $n_hi = BlobN($level+1) - 1;

  $Printwrap = 2;
  printwrap("\\node (start) at (0,0) [my dot];\n");
  printwrap("\\draw[my line] (start)");
  $Printwrap += 2;
  foreach my $n ($n_lo+1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    my ($tdx,$tdy) = $path->n_to_dxdy($n);
    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($tdx);
    my $ty = $y . chamfer($tdy);
    my ($xp,$yp) = $path->n_to_xy($n-1);
    my $seg = ($maxes{"$xp,$yp,$x,$y"} ? '' : '--');
    printwrap("$seg($sx,$sy)");
    printwrap("--($tx,$ty)");
  }
  my ($x,$y) = $path->n_to_xy($n_hi);
  my ($dx,$dy) = $path->n_to_dxdy($n_hi);
  $x += $dx/2;
  $y += $dy/2;
  printwrap("--($x,$y);\n");
  $Printwrap = 2;
  print "  \\coordinate (end) at ($x,$y);\n";
}

$Printwrap = 2;
foreach my $k ($dirmax_start_level .. $level) {
  my $n = DirMaxN($k);
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my $key = "$x,$y,$x2,$y2";
  ### %maxes
  ### $key
  if ($maxes{$key}) {
    delete $maxes{$key};
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my $mx = $x + $dx/2;
    my $my = $y + $dy/2;
    my $dir = dir($n);
    if ($dir < 0) {
      $dir = 'neg'.-$dir;
    }
    printwrap("\\coordinate (max-$dir) at ($mx,$my);");
    $x .= chamfer($dx);
    $y .= chamfer($dy);
    $x2 .= chamfer(-$dx);
    $y2 .= chamfer(-$dy);
    printwrap("\\draw[my maxes] ($x,$y)--($x2,$y2);");
  }
}
printwrap_newline();

{
  my $len = 11;
  print "% GP-Test  vector($len,k,k--; DirMaxN(k)) == [";
  foreach my $k (0 .. $len-1) { print $k==0?'':',', DirMaxN($k); }
  print "]\n";
}

print "  % end generated\n";
exit 0;
