#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.005;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','rect_corners_sort',
  'BlobN','Lpred_k','Rpred','JN';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;
my (undef,$n_hi) = $path->level_to_n_range($level);
my $n_lo = BlobN($level+1) - 3;

print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyLevelSubTwo}{",$level-2,"}\n";
print "  \\newcommand{\\MyLevelSubThree}{",$level-3,"}\n";
{
  my ($x,$y) = $path->n_to_xy($n_lo);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

my $JN = JN($level);

# segment immediately before curve end blob k-3
my $BlobN = $n_hi - (BlobN($level-2) - 2);
unless (Lpred_k($level,$BlobN) && Rpred($level,$BlobN)) {
  die "oops, not a bridge";
}
if (Lpred_k($level,$BlobN+1) && Rpred($level,$BlobN+1)) {
  die "oops, next is a bridge too";
}


my %seen;  # boundary squares
my %squares;

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  ### $n
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  $segments->add($x,$y, $x2,$y2);

  if (Lpred_k($level,$n) && $n >= $JN) {
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
    my ($xc,$yc) = rect_corners_sort ($x,$y, $x2+$lx,$y2+$ly);
    if (! $seen{"$xc,$yc"}++) {
      my $type = ($n < $BlobN ? 'LQ' : 'JA');
      push @{$squares{$type}}, "$xc,$yc";
    }
  }
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

foreach my $side ('LQ','JA') {
  my $aref = $squares{$side};
  $Printwrap = 2;
  printwrap("\\fill[my ${side} colour] foreach \\p in {");
  $Printwrap += 2;
  foreach my $xy (@$aref) {
    printwrap("($xy),");
  }
  printwrap("}\n");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

my $MyJAblobRQ = scalar(@{$squares{'LQ'}});
my $PreBlobRQ  = scalar(@{$squares{'JA'}});
print "  \\newcommand{\\MyPreBlobLQ}{$PreBlobRQ}\n";
print "  \\newcommand{\\MyJAblobRQ}{$MyJAblobRQ}\n";
print "% GP-Test  JA($level) == $MyJAblobRQ + $PreBlobRQ\n";
print "% GP-Test  PreBlobRQ($level-2) == $PreBlobRQ\n";

print "  % end generated\n";
exit 0;
