#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_to_node_name','chamfer','R','RJ';
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyLevelPlusOne}{",$level+1,"}\n";
my $RJ = RJ($level);
print "  \\newcommand{\\MyRJ}{$RJ}\n";
my $R = R($level);
print "  \\newcommand{\\MyR}{$R}\n";
my $RplusOne = R($level+1);
print "  \\newcommand{\\MyRplusOne}{",R($level+1),"}\n";
$Printwrap = 2;

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);

my %names;
foreach my $n (16, 20, 26, 32) {
  my ($x,$y) = $path->n_to_xy($n);
  my $name = xy_to_node_name($x,$y);
  $names{"$x,$y"} = $name;
  printwrap(" \\node ($name) at ($x,$y) [my dot];");
}
sub name {
  my ($x,$y) = @_;
  return $names{"$x,$y"} // "$x,$y";
}

my ($prev_x,$prev_y);
my @joins;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($dx1,$dy1) = $path->n_to_dxdy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx2,$dy2) = $path->n_to_dxdy($n+1);
  my $arrow = ($n & 1 ? '<-' : '->');

  my $xy1 = $names{"$x1,$y1"};
  if (! defined $xy1) {
    $x1 .= chamfer($dx1);
    $y1 .= chamfer($dy1);
    $xy1 = "$x1,$y1";
    if (defined $prev_x) {
      push @joins, "($prev_x,$prev_y)--($x1,$y1)";
    }
  }

  my $xy2 = $names{"$x2,$y2"};
  if (! defined $xy2) {
    $x2 .= chamfer(-$dx1);
    $y2 .= chamfer(-$dy1);
    $xy2 = "$x2,$y2";
    ($prev_x,$prev_y) = ($x2,$y2);
  } else {
    undef $prev_x;
    undef $prev_y;
  }
  printwrap(" \\draw[$arrow] ($xy1)--($xy2);");
}
printwrap_newline();

$Printwrap = 2;
printwrap("\\draw[line cap=round]");
$Printwrap += 2;
foreach my $join (@joins) {
  printwrap(" $join");
}
printwrap(";\n");

print "% GP-Test  RJ($level) == $RJ\n";
print "% GP-Test  R($level) == $R\n";
print "% GP-Test  R($level+1) == $RplusOne\n";
print "  % end generated\n";
exit 0;
