#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.006;
use strict;
use warnings;
use Carp 'croak';
use Math::PlanePath::ComplexPlus 117; # v.117 for level_to_n_range()
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh';

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'RQ','TAparent','TAtospine',
  'TAVertexToUpper','TAVertexToLower',
  'TAVertexToLeft','TAVertexToRight';
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexPlus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);


sub TA_n_is_spine {
  my ($n) = @_;
  return TAparent(TAtospine($n)) == $n;
}

$Printwrap = 2;
my @spine;
print "  \\begin{scope}[font=\\scriptsize]\n";
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  if (TA_n_is_spine($n)) {
    printwrap(" \\node (p$n) at ($x,$y) [my node] {\$$n\$};");
    push @spine, $n;
  } else {
    printwrap(" \\node (p$n) at ($x,$y) [my dot];");
  }
}
printwrap_newline();
print "  \\end{scope}\n";

my $count_spine = scalar(@spine);
my $rq = RQ($level);
$count_spine == $rq or die "oops, spine $count_spine, but RQ=$rq";
print "% GP-Test  vecsort(vector($count_spine,m,m--; TAspine(m))) == [",
  join(',',sort {$a<=>$b} @spine),"]\n";

my $num_edges = 0;
foreach my $want_spine (0, 1) {
  $Printwrap = 2;
  printwrap("\\draw".($want_spine ? '[my spine]' : '[my other]'));
  $Printwrap += 2;
  foreach my $from ($n_lo .. $n_hi) {
    foreach my $to (TAVertexToLeft($from),
                    TAVertexToRight($from),
                    TAVertexToLower($from),
                    TAVertexToUpper($from)) {
      if (defined $to
          && $to <= $n_hi
          && $to > $from
          && (TA_n_is_spine($from) && TA_n_is_spine($to)) == $want_spine) {
        printwrap(" (p$from)--(p$to)");
        $num_edges++;
      }
    }
  }
  printwrap(";\n");
}
$num_edges == 2**$level - 1
  or die "oops, num_edges=$num_edges";

print "  % end generated\n";
exit 0;
