#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';


# 3 8
# 4 8
my $arms = 3;
my $level = 8;

if (@ARGV) {
  @ARGV == 2 or die;
  ($arms,$level) = @ARGV;
}

print "  % generated by tools/$FindBin::Script $arms $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $arm (0 .. $arms-1) {

  {
    my ($x,$y) = ($x_hi,$y_hi);
    foreach (1 .. $arm) {
      ($x,$y) = xy_rotate_plus90($x,$y);
    }
    print "  \\node (end-$arm) at ($x,$y) [my dot];\n";
  }

  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    $x = $x_hi - $x;
    $y = $y_hi - $y;
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $x2 = $x_hi - $x2;
    $y2 = $y_hi - $y2;

    foreach (1 .. $arm) {
      ($x,$y) = xy_rotate_plus90($x,$y);
      ($x2,$y2) = xy_rotate_plus90($x2,$y2);
    }
    $segments->add($x,$y, $x2,$y2);
  }
}
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
