#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyPlanar;
use MyDragon 'BlobN',
  'points_to_hull','points_to_bigrat','transform_to_horizontal';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 16;
if (@ARGV) { $max_level = shift @ARGV; }
$|=1;

print "% generated by tools/$FindBin::Script\n";

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;
print "% GP-DEFINE  $var = {[\n";

my $path = Math::PlanePath::DragonCurve->new;

sub xy_to_expression {
  my ($x,$y) = @_;

  if ($x == 0 && $y == 0) {
    return '0';
  }
  my $str = '';
  if ($x) {
    $str = $x;
  }
  if ($y) {
    if ($y > 0 && length($str)) { $str .= '+'; }
    $str .= "$y*I";
  }
  return $str;
}

my @BlobHA;
foreach my $k (4 .. $max_level) {
  my $n_start = BlobN($k);
  my $n_end = BlobN($k+1)-3;

  my $points = [];
  foreach my $n ($n_start .. $n_end) {
    my ($x,$y) = $path->n_to_xy($n);
    push @$points, [$x,$y];
  }

  $points = points_to_hull($points);
  # @$points = reverse @$points;  # clockwise

  {
    # rotate to BP1 first in $points
    my $transformed_points = [ map {[transform_to_horizontal(@$_,$k)]} @$points ];
    my $BP1_y = min(map {$_->[1]} @$transformed_points);
    my $BP1_x = max(map {$_->[0]} grep {$_->[1]==$BP1_y} @$transformed_points);
    my ($BP1_i) = grep {$transformed_points->[$_]->[0] == $BP1_x
                          && $transformed_points->[$_]->[1] == $BP1_y
                        } 0 .. $#$transformed_points
                        or die;
    ### BP1 at: "$BP1_x, $BP1_y  i=$BP1_i"
    $points = [ map {$points->[($BP1_i-$_)%scalar(@$points)]} 0 .. $#$points ];
  }

  if ($k >= 9 && !($k%4==2 || $k%4==3)) {
    # point 4 at i=3, insert copy as 4'
    splice @$points, 3,0, $points->[3];
  }

  if ($k >= 9 && !($k%4==0 || $k%4==3)) {
    # point 8 at i=8, insert copy as 8'
    splice @$points, 8, 0, $points->[8];
  }

  print "% GP-DEFINE    /* $k */ [ ";
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    print $join, xy_to_expression($x,$y);
    $join = ',';
  }
  print " ]", ($k<$max_level ? "," : ""), "\n";
  push @BlobHA, MyPlanar::points_to_area(points_to_bigrat($points));
}

print "% GP-DEFINE  ]};\n";
print "% GP-DEFINE  ${var}_BlobHA = [",join(',',@BlobHA),"];\n";
print "% end generated\n";
exit 0;






__END__

  # my ($sx,$sy) = BlobStart_xy($k);
  # my $i = points_find_nearest_i($points, $sx,$sy);
  # if ($k >= 9 && !($k%4==2 || $k%4==3)) {
  #   # point 4a' copy
  #   splice @$points, $i, 0, $points->[$i];
  # }
  # # $i is BP4, want at index 5
  # $points = [ map {$points->[($_-$i+5)%scalar(@$points)]} 0 .. $#$points ];


  # sub points_find_nearest_i {
  #   my ($points, $x,$y) = @_;
  #   my $min_i = 0;
  #   my $min_norm;
  #   foreach my $i (0 .. $#$points) {
  #     my $p = $points->[$i];
  #     my $norm = ($x - $p->[0])**2 + ($x - $p->[1])**2;
  #     if (! defined $min_norm || $norm < $min_norm) {
  #       $min_i = $i;
  #       $min_norm = $norm;
  #     }
  #   }
  #   return $min_i;
  # }
