#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN', 'BlobNend';
use MyLineSegments;
use MyPrintwrap ':all';


my $level_start = 6;
my $level_end   = 7;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevelStart}{$level_start}\n";
print "  \\newcommand{\\MyLevelEnd}{$level_end}\n";

my $path = Math::PlanePath::DragonCurve->new;
my $n_start = BlobN($level_start) - 1;
my $n_end   = BlobNend($level_end) + 1;    # point number

{
  my ($x,$y) = $path->n_to_xy($n_start);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_end);
  print "  \\coordinate (end) at ($x,$y);\n";
}

MyDragon::fBpred_surround_init();
my $f = $MyDragon::fNonBpred_surround_flat;

my $fNonB = 0;
$Printwrap = 2;
foreach my $n ($n_start .. $n_end-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  if ($n & 1) {
    ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
  }
  my $n2 = sprintf '%b', $n;
  my $style = 'my arrow';
  if ($f->contains($n2)) {
    $style ='my thick arrow';
    $fNonB++;
  }
  printwrap(" \\draw[$style] ($x1,$y1)--($x2,$y2);");
}
printwrap_newline();

print "% GP-Test-Last  fNonB($level_end) == $fNonB\n";
print "  % end generated\n";
exit 0;
