#!/usr/bin/perl -w

# Copyright 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\node (start) at (0,0) [my dot];\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

my $y_hi = 3;
my $y_lo = -3;
sub is_y_in_range {
  my ($y) = @_;
  return $y_lo <= $y && $y <= $y_hi;
}

# Verticals extend 1/4 of a segment beyond the y_lo,y_hi limits.
my $scale = 4;
my $y_scaled_hi = $y_hi * $scale + 1;
my $y_scaled_lo = $y_lo * $scale - 1;
sub y_scaled_truncate {
  my ($y) = @_;
  return max($y_scaled_lo, min($y_scaled_hi, $y));
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  if (is_y_in_range($y1) || is_y_in_range($y2)) {
    $y1 *= $scale;
    $y2 *= $scale;
    $y1 = y_scaled_truncate($y1);
    $y2 = y_scaled_truncate($y2);
    $segments->add($x1,$y1, $x2,$y2);
  }
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        $y /= $scale;
                        return ($x,$y);
                      });
printwrap(";\n");

print "  % end generated\n";
