#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Lingua::EN::Numbers;
use Math::PlanePath::CCurve 118; # v.118 for xyxy_to_n()
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_to_dir4','xy_rotate_plus90';
use MyPrintwrap ':all';


my $level = 4;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

foreach my $pos (0, 1) {
  my $path = ($pos == 0
              ? Math::PlanePath::DragonCurve->new
              : Math::PlanePath::CCurve->new);

  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$pos)]\n");

  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my @count = (0,0,0,0);
  $Printwrap = 3;
  foreach my $n ($n_lo .. $n_hi-1) {
    my $options = ($n == $n_lo ? 'shorten <=0pt,' : '');

    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    my $dx = $x2-$x1;
    my $dy = $y2-$y1;

    my $dir4 = xy_to_dir4($dx,$dy);
    $count[$dir4]++;

    if (defined $path->xyxy_to_n($x2,$y2, $x1,$y1)) {
      # line segment traversed both ways
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      printwrap(" \\draw[${options}-{>[harpoon]}] ($x1+\\MySep*$dx,$y1+\\MySep*$dy)--($x2+\\MySep*$dx,$y2+\\MySep*$dy);");

    } else {
      # line segment traversed one way only
      printwrap(" \\draw[${options}->] ($x1,$y1)--($x2,$y2);");
    }
  }
  printwrap_newline();
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
  foreach my $d (0 .. $#count) {
    print "% GP-Test  Skd($level,$d)==$count[$d]\n";
  }
  if ($pos == 0) {
    foreach my $d (0 .. $#count) {
      my $name = Lingua::EN::Numbers::num2en($d);
      printwrap("\\newcommand{\\MyS$name}{$count[$d]}\n");
    }
  }
}

print "  % end generated\n";
exit 0;
