#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;
print "  % generated by tools/$FindBin::Script\n";
print "  % level $level\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

# {
#   my $segments = MyLineSegments->new;
#   foreach my $n ($n_lo .. $n_hi-1) {
#     my ($x,$y) = $path->n_to_xy($n);
#     $segments->add($x,$y, $path->n_to_xy($n+1));
#   }
#   $Printwrap = 2;
#   printwrap("\\draw[very thin,draw=my grey]");
#   $Printwrap = 3;
#   foreach my $line ($segments->lines) {
#     my ($x1,$y1, $x2,$y2) = @$line;
#     printwrap(" ($x1,$y1)--($x2,$y2)");
#   }
#   printwrap(";\n");
# }

{
  $Printwrap = 2;
  printwrap("\\fill foreach \\p in {");

  my $x_max = 0;
  my $count = 0;
  $Printwrap = 4;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    @n_list = grep {$_ <= $n_hi} @n_list;
    next if @n_list > 1;
    printwrap("($x,$y),");
    $count++;
    $x_max = max($x_max, $x);
  }
  printwrap("}\n");
  print "    { \\p circle[] };\n";

  print "  \\coordinate (x-max) at ($x_max,0);\n";
  print "  \\newcommand{\\MyLevel}{$level}\n";
  print "  \\newcommand{\\MyCount}{$count}\n";
  print "% GP-Test-last  S($level) == $count\n";
}

print "  % end generated\n";
exit 0;
