#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()
use Tie::IxHash;

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@dir4_to_dx','@dir4_to_dy','xy_rotate_plus90','mul','div';
use MyLineSegments;
use MyPrintwrap ':all';


my $max_level = 6;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;

my @TP;
my @TD;
my @TS;
my @TPB;
my @TPE;
my @TDB;

foreach my $k (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  my $twindragon_xy_num_visits = sub {
    my ($x,$y) = @_;
    my @n_list = ($path->xy_to_n_list($x,$y),
                  $path->xy_to_n_list($x_hi-$x,$y_hi-$y));
    @n_list = grep {$_ < $n_hi} @n_list;
    return scalar @n_list;
  };

  my $twindragon_xyxy_is_traversed = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    my @n_list = ($path->xyxy_to_n_list_either($x1,$y1, $x2,$y2),
                  $path->xyxy_to_n_list_either($x_hi-$x1,$y_hi-$y1,
                                               $x_hi-$x2,$y_hi-$y2));
    @n_list = grep {$_ < $n_hi} @n_list;
    return scalar @n_list;
  };

  #   * --- * --- *
  #   |     |     |
  #   * -- x,y -- *
  #   |     |     |
  #   * --- * --- *

  my $twindragon_xy_is_enclosed = sub {
    my ($x,$y) = @_;
    foreach my $dx (-1, 0) {        # horizontals
      foreach my $dy (-1, 0, 1) {
        if (! $twindragon_xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx+1,$y+$dy)) {
          return 0;
        }
      }
    }
    foreach my $dy (-1, 0) {        # verticals
      foreach my $dx (-1, 0, 1) {
        if (! $twindragon_xyxy_is_traversed->($x+$dx,$y+$dy, $x+$dx,$y+$dy+1)) {
          return 0;
        }
      }
    }
    return 1;
  };

  my $TP = 0;
  my $TD = 0;
  my $TS = 0;
  my $TPB = 0;
  my $TPE = 0;
  my $TDB = 0;
  my %seen;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    foreach my $rev (0, 1) {
      if ($rev) { ($x,$y) = ($x_hi - $x, $y_hi - $y); }
      next if $seen{"$x,$y"}++;

      $TP++;
      my $num_visits = $twindragon_xy_num_visits->($x,$y);
      if ($num_visits == 1) { $TS++; }
      if ($num_visits == 2) { $TD++; }

      my $enclosed = $twindragon_xy_is_enclosed->($x,$y);
      my $boundary = ! $enclosed;
      if ($enclosed) { $TPE++; }
      if ($boundary) { $TPB++; }
      if ($boundary && $num_visits == 2) { $TDB++; }
    }
  }

  push @TP,  $TP;
  push @TD,  $TD;
  push @TS,  $TS;
  push @TPB, $TPB;
  push @TPE, $TPE;
  push @TDB, $TDB;
}

print "% GP-DEFINE  TP_generated_samples = [ ",join(',',@TP), " ];\n";
print "% GP-DEFINE  TD_generated_samples = [ ",join(',',@TD), " ];\n";
print "% GP-DEFINE  TS_generated_samples = [ ",join(',',@TS), " ];\n";
print "% GP-DEFINE  TPB_generated_samples = [ ",join(',',@TPB), " ];\n";
print "% GP-DEFINE  TPE_generated_samples = [ ",join(',',@TPE), " ];\n";
print "% GP-DEFINE  TDB_generated_samples = [ ",join(',',@TDB), " ];\n";

print "% end generated\n";
exit 0;
