#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::Complex;
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'points_str','hull_points_sort_by_angle',
  'P1_xy','P2_xy','P3_xy','P4_xy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 7;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);


$Printwrap = 2;
printwrap("\\coordinate (end-point) at ($x_hi,$y_hi);\n");

# curve
my $curve_points = [];
{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    foreach (1, 2) {
      $segments->add($x1,$y1, $x2,$y2);
      push @$curve_points, [$x1,$y1];

      $x1 = $x_hi - $x1;
      $y1 = $y_hi - $y1;
      $x2 = $x_hi - $x2;
      $y2 = $y_hi - $y2;
    }
  }

  $Printwrap = 2;
  printwrap("\\draw[my dragon line] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

# hull
my $hull_points = [];
{
  my @vertices = (['P1', P1_xy($level+1)],
                  ['P2', P2_xy($level+1)],
                  ['P3', P3_xy($level+1)],
                  ['P4', P4_xy($level+1)]);
  push @vertices, map {
    my ($name, $x,$y) = @$_;
    $name .= 'o';
    ($x,$y) = ($x_hi-$x,$y_hi-$y);
    [$name, $x,$y];
  } @vertices;

  foreach my $elem (@vertices) {
    my ($name, $x,$y) = @$elem;
    push @$hull_points, [$x,$y];
    print "  \\node ($name) at ($x,$y) [my hull dot];\n";
  }
  foreach my $elem (@vertices) {
    my ($name, $x,$y) = @$elem;
    print "% GP-Test  T_$name($level) == $x+I*$y\n";
  }
}

my $planar = Math::Geometry::Planar->new;
my $points = $curve_points;
$planar->points($points);
if (@$curve_points >= 5) {
  $planar = $planar->convexhull2;
  $points = $planar->points;
}
my $area = $planar->area;
print "  \\newcommand{\\MyArea}{$area}\n";
print "% GP-Test-Last  THA($level) == $area\n";

my $planar_points = $planar->points;

{
  # planar and formulas should be same
  $hull_points   = hull_points_sort_by_angle($hull_points,   $x_hi,$y_hi);
  $planar_points = hull_points_sort_by_angle($planar_points, $x_hi,$y_hi);

  my $hull_points_str = points_str($hull_points);
  my $planar_points_str = points_str($planar_points);
  unless ($hull_points_str eq $planar_points_str) {
    print "$hull_points_str\n";
    print "$planar_points_str\n";
    die;
  }
}

print "  % end generated\n";
exit 0;
