#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()
use Math::PlanePath::DragonMidpoint 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

{
  # dragon curve

  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  $Printwrap = 2;
  {
    my ($x,$y) = $path->n_to_xy($n_lo);
    printwrap("\\node (start) at ($x,$y) [my dot];\n");
  }
  printwrap("\\node (end) at ($x_hi,$y_hi) [my dot];\n");

  my $twindragon_n_to_xy = sub {
    my ($n) = @_;
    if ($n < $n_hi) {
      return $path->n_to_xy($n);
    } else {
      my ($x,$y) = $path->n_to_xy($n - $n_hi);
      return ($x_hi - $x, $y_hi - $y);
    }
  };
  my $twindragon_n_to_dxdy = sub {
    my ($n) = @_;
    if ($n < $n_hi) {
      return $path->n_to_dxdy($n);
    } else {
      my ($dx,$dy) = $path->n_to_dxdy($n - $n_hi);
      return (-$dx, -$dy);
    }
  };

  $Printwrap = 2;
  printwrap("\\draw[my line] ");
  $Printwrap = 4;
  my $join = '';
  foreach my $n ($n_lo .. 2*$n_hi-1) {
    my ($x,$y) = $twindragon_n_to_xy->($n);
    my ($dx,$dy) = $twindragon_n_to_dxdy->($n);

    my ($x2,$y2) = $twindragon_n_to_xy->($n+1);

    my $sx = $x . chamfer($dx);
    my $sy = $y . chamfer($dy);
    my $tx = $x2 . chamfer(- $dx);
    my $ty = $y2 . chamfer(- $dy);
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");
    $join = '--';
  }
  printwrap("--cycle;\n");
}

{
  # walls

  my $path = Math::PlanePath::DragonMidpoint->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi+1);

  my $segments = MyLineSegments->new;
  foreach my $rotate (0,1) {
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      if ($rotate) {
        ($x,$y) = ($x_hi-$x, $y_hi-$y);
      }

      $segments->add($x,$y, $x+1,$y);
      $segments->add($x,$y+1, $x+1,$y+1);
      $segments->add($x,$y, $x,$y+1);
      $segments->add($x+1,$y, $x+1,$y+1);

      if ($n >= 1) {
        my ($dx,$dy) = $path->n_to_dxdy($n-1);
        if ($rotate) {
          ($dx,$dy) = (-$dx,-$dy);
        }
        if ($dx == 0) {
          if ($dy < 0) { $y += 1; }
          $segments->remove($x,$y, $x+1,$y);
        } else {
          if ($dx < 0) { $x += 1; }
          $segments->remove($x,$y, $x,$y+1);
        }
      }
    }
  }
  {
    my ($x,$y) = $path->n_to_xy($n_lo);
    $segments->remove($x,$y+1, $x+1,$y+1);
    ($x,$y) = ($x_hi-$x, $y_hi-$y);
    $segments->remove($x,$y, $x+1,$y);
  }


  # {
  #   # open end
  #   my $n = $n_hi + 1;
  #   my ($x,$y) = $path->n_to_xy($n);
  #   my ($dx,$dy) = $path->n_to_dxdy($n-1);
  #   if ($dx == 0) {
  #     if ($dy < 0) { $y += 1; }
  #     $segments->remove($x,$y, $x+1,$y);
  #   } else {
  #     if ($dx < 0) { $x += 1; }
  #     $segments->remove($x,$y, $x,$y+1);
  #   }
  # }

  my $transform = sub {
    my ($x,$y) = @_;
    ($x,$y) = mul($x,$y);
    return (($x+1)/2, ($y-1)/2);
  };

  $Printwrap = 2;
  printwrap("\\draw[my wall]");
  $Printwrap = 4;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    ($x1,$y1) = $transform->($x1,$y1);
    ($x2,$y2) = $transform->($x2,$y2);
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");

  my @segs = $segments->segments;
  my $count = scalar(@segs);
  print "  \\newcommand{\\MyCount}{$count}\n";
  print "% GP-Test-Last  TW($level)==$count\n";
}

print "  % end generated\n";
exit 0;
