#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

print "% generated by tools/$FindBin::Script\n";


{
  my $path = Math::PlanePath::DragonCurve->new;

  my @count_odd_odd;
  my @count_odd_even;
  my @count_even_even;
  foreach my $k (1 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    ### $k
    ### $n_hi

    my %seen;
    my $count_odd_odd = 0;
    my $count_odd_even = 0;
    my $count_even_even = 0;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      for (1, 2) {
        next if $seen{"$x,$y"}++;
        my @n_list = ($path->xy_to_n_list($x,$y),
                      $path->xy_to_n_list($x_hi-$x, $y_hi-$y));
        ### @n_list
        @n_list = grep {$_ < $n_hi} @n_list;
        my $c = scalar(@n_list);
        $c == 1 || $c == 2 || die;
        if ($c == 1) {
          if (($x&1)==0 && ($y&1)==0) {
            $count_even_even++;
          } elsif (($x&1)==1 && ($y&1)==0) {
            $count_odd_even++;
          } elsif (($x&1)==1 && ($y&1)==1) {
            $count_odd_odd++;
          } else {
            die "oops n=$n c=$c  $x, $y";        # even,odd should not occur
          }
        }

        $x = $x_hi - $x;
        $y = $y_hi - $y;
      }
    }
    push @count_odd_odd, $count_odd_odd;
    push @count_odd_even, $count_odd_even;
    push @count_even_even, $count_even_even;
  }
  print "% GP-DEFINE  gen_TS_odd_odd = [ ",join(',',@count_odd_odd),"];\n";
  print "% GP-DEFINE  gen_TS_odd_even = [ ",join(',',@count_odd_even),"];\n";
  print "% GP-DEFINE  gen_TS_even_even = [ ",join(',',@count_even_even),"];\n";
}

{
  my $path = Math::PlanePath::ComplexMinus->new;

  my @count_a;
  my @count_b;
  my @count_c;
  foreach my $k (1 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    my $count_a = 0;
    my $count_b = 0;
    my $count_c = 0;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $odd = ($x+$y)%2;
      my $even = ! $odd;
      my $n;

      # below
      if (defined ($n = $path->xy_to_n($x,$y-1))
          && $n <= $n_hi) {
        ### not on boundary ...
      } else {
        if ($odd) { $count_b++; } else { $count_a++; }
      }

      # above
      if (defined ($n = $path->xy_to_n($x,$y+1))
          && $n <= $n_hi) {
        ### not on boundary ...
      } else {
        if ($odd) { $count_a++; } else { $count_b++; }
      }

      # left
      if ($even) {
        if (defined ($n = $path->xy_to_n($x-1,$y))
            && $n <= $n_hi) {
          ### not on boundary ...
        } else {
          $count_c++;
        }
      }

      # right
      if ($odd) {
        if (defined ($n = $path->xy_to_n($x+1,$y))
            && $n <= $n_hi) {
          ### not on boundary ...
        } else {
          $count_c++;
        }
      }
    }
    $count_a /= 2;
    $count_b /= 2;
    $count_c /= 2;
    push @count_a, $count_a;
    push @count_b, $count_b;
    push @count_c, $count_c;
  }
  print "% GP-DEFINE  gen_TS_a_dash_samples = [ ",join(',',@count_a),"];\n";
  print "% GP-DEFINE  gen_TS_b_dash_samples = [ ",join(',',@count_b),"];\n";
  print "% GP-DEFINE  gen_TS_c_dash_samples = [ ",join(',',@count_c),"];\n";
}

print "% end generated\n";
exit 0;


