#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::BigRat;

use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) {
  $level = shift @ARGV;
}

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my @points = ([Math::BigRat->new('1/5'), Math::BigRat->new('-3/5'), 2],
              [Math::BigRat->new('4/5'), Math::BigRat->new('-2/5'), 1],
              [Math::BigRat->new('6/5'), Math::BigRat->new('2/5'), 0],
              [Math::BigRat->new('4/5'), Math::BigRat->new('8/5'), 2],
              [Math::BigRat->new('1/5'), Math::BigRat->new('7/5'), 1],
              [Math::BigRat->new('-1/5'), Math::BigRat->new('3/5'), 0],
             );
# ,
#               [0, 0, 0],
#               [Math::BigRat->new(1), 0]

my $expand_norm = Math::BigRat->new('2/5')**2 + Math::BigRat->new('6/5')**2;
foreach my $k (1 .. $level) {
  my @new_points;
  foreach my $i (0 .. $#points) {
    my ($x1,$y1, $expansion) = @{$points[$i]};
    my ($x2,$y2)             = @{$points[($i+1) % scalar(@points)]};

    my $norm = ($x1-$x2)**2 + ($y1-$y2)**2;
    if ($norm < $expand_norm) {
      push @new_points, [$x1, $y1, $expansion];

    } else {
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;
      my $dx4 = $dx/4;
      my $dy4 = $dy/4;
      my ($lx4,$ly4) = xy_rotate_plus90($dx4, $dy4);  # first to left

      push @new_points, [$x1, $y1,
                         $expansion + 3];
      push @new_points, [$x1 + $dx4 + $lx4,
                         $y1 + $dy4 + $ly4,
                         $expansion + 1];
      push @new_points, [$x2 - $dx4 - $lx4,
                         $y2 - $dy4 - $ly4,
                         $expansion + 3];
    }
  }

  # push @new_points, $points[-1];
  @points = @new_points;
  $expand_norm /= 2;
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
my $join = '';
foreach my $p (@points) {
  my ($x,$y) = @$p;
  printwrap("$join($x,$y)");
  $join = '--';
}
printwrap("${join}cycle;\n");

my $count = scalar(@points);
print "  % $count points\n";
print "  % end generated\n";
exit 0;
