#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use File::stat;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use Math::PlanePath::GcdRationals;
*gcd = \&Math::PlanePath::GcdRationals::_gcd;

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';


my $path = Math::PlanePath::DragonCurve->new;

foreach my $level (1 .. 5) {
  print "level $level\n";
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  my $points = [ ];
  foreach my $n ($n_lo .. $n_hi) {
    push @$points, [$path->n_to_xy($n)];
  }

  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  if (@$points >= 5) {
    $planar = $planar->convexhull2;
  }
  my $hull_points = $planar->points;
  print "  hull vertices ",scalar(@$hull_points),"\n";

  my %seen;
  foreach my $i (0 .. $#$hull_points) {
    my $dx = ($hull_points->[$i]->[0] - $hull_points->[$i-1]->[0]);
    my $dy = ($hull_points->[$i]->[1] - $hull_points->[$i-1]->[1]);
    if (abs($dx) == 0) {
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    }
    my $slope = $dy / $dx;
    next if $seen{$slope};
    $slope = - $slope;
    next if $seen{$slope}++;
    print "  rot $dx $dy   slope=$slope\n";

    $dy = - $dy;
    # (x+i*y) * (dx + i*dy)
    #  = x*dx - y*dy
    #    + (x*dy + y*dx) * i
    my $x_min = 0;
    my $y_min = 0;
    my $x_max = 0;
    my $y_max = 0;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $rx = $x * $dx - $y * $dy;
      my $ry = $x * $dy + $y * $dx;
      $x_min = min($x_min, $rx);
      $x_max = max($x_max, $rx);
      $y_min = min($y_min, $ry);
      $y_max = max($y_max, $ry);
    }
    print "    extents $x_min $x_max and $y_min $y_max\n";

    my $width = $x_max - $x_min;
    my $height = $y_max - $y_min;
    print "    size $width x $height\n";
    my $area = $width * $height;
    my $norm = $dx*$dx + $dy * $dy;

    my $g = gcd($area, $norm);
    $area /= $g;
    $norm /= $g;
    my $r = $area % $norm;
    my $q = ($area - $r) / $norm;
    print "    area $area / $norm                        = $q + $r/$norm\n";
  }
}
exit 0;
