#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'ffixed_du', 'ffixed_u_parent', 'ffixed_row_w';
use MyLineSegments;
use MyPrintwrap ':all';

my $max_row = 6;

print "  % generated by tools/$FindBin::Script\n";

sub ffixed_from_index {
  my ($m) = @_;
  my $r = $m % 3;
  my $h = ($m-$r)/3;
  if ($r == 0) { return 2*$h; }
  if ($r == 1) { return undef; }
  return 2*$h+1;
}

my %nodes;
$Printwrap = 2;
foreach my $row (0 .. $max_row) {
  my $max_m = ($row==0 ? 0 : ffixed_row_w($row-1));
  foreach my $m (0 .. $max_m) {
    my $du = ffixed_du($m);
    my $node = "row$row-$m";
    printwrap(" \\node ($node) at ($node) [my box] {\$$du\$};");
    $nodes{$node} = 1;
  }
  printwrap_newline();

  if ($row >= 1) {
    my $prev_row = $row-1;
    foreach my $m (0 .. $max_m) {
      last if $row == 1 && $m == 1;
      my $parent = ffixed_u_parent($m);
      my $parent_node = "row$prev_row-$parent";
      exists $nodes{$parent_node} or die "oops, no parent $parent_node";
      my $style = ($m%3 == 1 ? 'dotted' : '->');
      printwrap(" \\draw[$style] ($parent_node) to (row$row-$m);");
    }
  }
  printwrap_newline();
}

print "% GP-Test  vector(",$max_row+1,",k,k--; ffixed_row_w(k)) == \\\n";
print "% GP-Test    [",join(',',map {ffixed_row_w($_)} 0 .. $max_row),"]\n";

my $max_w = ffixed_row_w($max_row);
print "% GP-Test  vector(",$max_w,",m,m--; ffixed_du(m)) == \\\n";
print "% GP-Test    [",join(',',map {ffixed_du($_)} 0 .. $max_w-1),"]\n";

print "  % end generated\n";
exit 0;
